/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickHAT;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the HAT Brick.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickHATActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set Sleep Mode")
    public void brickHATSetSleepMode(@ActionInput(name = "powerOffDelay") long powerOffDelay,
            @ActionInput(name = "powerOffDuration") long powerOffDuration,
            @ActionInput(name = "raspberryPiOff") boolean raspberryPiOff,
            @ActionInput(name = "brickletsOff") boolean brickletsOff,
            @ActionInput(name = "enableSleepIndicator") boolean enableSleepIndicator) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickHATWrapper dev = ((@Nullable BrickHATWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setSleepMode(powerOffDelay, powerOffDuration, raspberryPiOff, brickletsOff, enableSleepIndicator);

    }

    public static void brickHATSetSleepMode(@Nullable ThingActions actions, long powerOffDelay, long powerOffDuration,
            boolean raspberryPiOff, boolean brickletsOff, boolean enableSleepIndicator) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            ((BrickHATActions) actions).brickHATSetSleepMode(powerOffDelay, powerOffDuration, raspberryPiOff,
                    brickletsOff, enableSleepIndicator);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Sleep Mode")
    public @ActionOutput(name = "powerOffDelay", type = "long") @ActionOutput(name = "powerOffDuration", type = "long") @ActionOutput(name = "raspberryPiOff", type = "boolean") @ActionOutput(name = "brickletsOff", type = "boolean") @ActionOutput(name = "enableSleepIndicator", type = "boolean") Map<String, Object> brickHATGetSleepMode()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickHATWrapper dev = ((@Nullable BrickHATWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickHAT.SleepMode value = dev.getSleepMode();

        result.put("powerOffDelay", value.powerOffDelay);
        result.put("powerOffDuration", value.powerOffDuration);
        result.put("raspberryPiOff", value.raspberryPiOff);
        result.put("brickletsOff", value.brickletsOff);
        result.put("enableSleepIndicator", value.enableSleepIndicator);
        return result;
    }

    public static Map<String, Object> brickHATGetSleepMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetSleepMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Set Bricklet Power")
    public void brickHATSetBrickletPower(@ActionInput(name = "brickletPower") boolean brickletPower)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickHATWrapper dev = ((@Nullable BrickHATWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setBrickletPower(brickletPower);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickHATPowerBricklets")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickHATPowerBricklets"), RefreshType.REFRESH);
        }
    }

    public static void brickHATSetBrickletPower(@Nullable ThingActions actions, boolean brickletPower)
            throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            ((BrickHATActions) actions).brickHATSetBrickletPower(brickletPower);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Bricklet Power")
    public @ActionOutput(name = "brickletPower", type = "boolean") Map<String, Object> brickHATGetBrickletPower()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickHATWrapper dev = ((@Nullable BrickHATWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.getBrickletPower();

        result.put("brickletPower", value);
        return result;
    }

    public static Map<String, Object> brickHATGetBrickletPower(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetBrickletPower();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Voltages")
    public @ActionOutput(name = "voltageUSB", type = "int") @ActionOutput(name = "voltageDC", type = "int") Map<String, Object> brickHATGetVoltages()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickHATWrapper dev = ((@Nullable BrickHATWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickHAT.Voltages value = dev.getVoltages();

        result.put("voltageUSB", value.voltageUSB);
        result.put("voltageDC", value.voltageDC);
        return result;
    }

    public static Map<String, Object> brickHATGetVoltages(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetVoltages();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickHATReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickHATWrapper dev = ((@Nullable BrickHATWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickHATReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickHATGetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickHATWrapper dev = ((@Nullable BrickHATWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickHATGetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickHATGetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickHATWrapper dev = ((@Nullable BrickHATWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickHATGetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickHATGetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickHATWrapper dev = ((@Nullable BrickHATWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickHAT.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickHATGetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "int[]") @ActionOutput(name = "firmwareVersion", type = "int[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickHATGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickHATWrapper dev = ((@Nullable BrickHATWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickHAT.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickHATGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }
}
