/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletAmbientLightV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Ambient Light Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAmbientLightV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Illuminance")
    public @ActionOutput(name = "illuminance", type = "long") Map<String, Object> brickletAmbientLightV2GetIlluminance()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightV2Wrapper dev = ((@Nullable BrickletAmbientLightV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.getIlluminance();

        result.put("illuminance", value);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightV2GetIlluminance(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightV2Actions) {
            return ((BrickletAmbientLightV2Actions) actions).brickletAmbientLightV2GetIlluminance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "illuminanceRange", type = "short") @ActionOutput(name = "integrationTime", type = "short") Map<String, Object> brickletAmbientLightV2GetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightV2Wrapper dev = ((@Nullable BrickletAmbientLightV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAmbientLightV2.Configuration value = dev.getConfiguration();

        result.put("illuminanceRange", value.illuminanceRange);
        result.put("integrationTime", value.integrationTime);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightV2GetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightV2Actions) {
            return ((BrickletAmbientLightV2Actions) actions).brickletAmbientLightV2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletAmbientLightV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightV2Wrapper dev = ((@Nullable BrickletAmbientLightV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAmbientLightV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightV2Actions) {
            return ((BrickletAmbientLightV2Actions) actions).brickletAmbientLightV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightV2Actions class.");
        }
    }
}
