/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletAnalogInV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Analog In Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletAnalogInV2Wrapper extends BrickletAnalogInV2 implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletAnalogInV2.DEVICE_DISPLAY_NAME,
            "brickletanaloginv2", BrickletAnalogInV2.DEVICE_IDENTIFIER, BrickletAnalogInV2Wrapper.class,
            BrickletAnalogInV2Actions.class, "2.0.0", false);

    public BrickletAnalogInV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletAnalogInV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletAnalogInV2Config cfg = (BrickletAnalogInV2Config) config.as(BrickletAnalogInV2Config.class);
        this.addVoltageListener(this.reg((voltage) -> {
            if (true) {
                updateStateFn.accept("BrickletAnalogInV2Voltage",
                        transformBrickletAnalogInV2VoltageCallback0(voltage, cfg));
            }
        }, this::removeVoltageListener));
        this.addAnalogValueListener(this.reg((value) -> {
            if (true) {
                updateStateFn.accept("BrickletAnalogInV2AnalogValue",
                        transformBrickletAnalogInV2AnalogValueCallback0(value, cfg));
            }
        }, this::removeAnalogValueListener));
        this.setMovingAverage(cfg.movingAverageLength.shortValue());
        {
            BrickletAnalogInV2VoltageConfig channelCfg = getChannelConfigFn.apply("BrickletAnalogInV2Voltage")
                    .as(BrickletAnalogInV2VoltageConfig.class);
            this.setVoltageCallbackPeriod(channelCfg.updateInterval);
            this.setVoltageCallbackThreshold('x', 0, 0);
        }
        {
            BrickletAnalogInV2AnalogValueConfig channelCfg = getChannelConfigFn.apply("BrickletAnalogInV2AnalogValue")
                    .as(BrickletAnalogInV2AnalogValueConfig.class);
            this.setAnalogValueCallbackPeriod(channelCfg.updateInterval);
            this.setAnalogValueCallbackThreshold('x', (short) 0, (short) 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setVoltageCallbackPeriod(0);
        this.setAnalogValueCallbackPeriod(0);

    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletAnalogInV2Voltage");
        result.add("BrickletAnalogInV2AnalogValue");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletAnalogInV2Voltage":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletAnalogInV2Voltage"), "Voltage",
                                "Number:ElectricPotential")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletAnalogInV2Voltage"))
                        .withDescription("Measured voltage")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(42)).withStep(BigDecimal.valueOf(0.001))
                                        .withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletAnalogInV2AnalogValue":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletAnalogInV2AnalogValue"), "Analog Value",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletAnalogInV2AnalogValue"))
                        .withDescription("<br/>Returns the value as read by a 12-bit analog-to-digital converter.<br/>")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(4095)).withStep(BigDecimal.valueOf(1))
                                        .withPattern("%d").withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Analog In Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Measures DC voltage between 0V and 42V")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletAnalogInV2Voltage",
                                new ChannelTypeUID("tinkerforge", "BrickletAnalogInV2Voltage")).withLabel("Voltage")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletAnalogInV2AnalogValue",
                                new ChannelTypeUID("tinkerforge", "BrickletAnalogInV2AnalogValue"))
                                        .withLabel("Analog Value").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletanaloginv2":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("movingAverageLength", Type.INTEGER).withDefault("50")
                                .withDescription(
                                        "The length of a moving averaging for the voltage.<br/><br/>Setting the length to 1 will turn the averaging off. With less averaging, there is more noise on the data.")
                                .withLabel("Moving Average Length").withAdvanced(true)
                                .withMinimum(BigDecimal.valueOf(1)).withMaximum(BigDecimal.valueOf(50)).build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletAnalogInV2Voltage":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletAnalogInV2AnalogValue":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletAnalogInV2Config cfg = (BrickletAnalogInV2Config) config.as(BrickletAnalogInV2Config.class);
        switch (channel) {
            case "BrickletAnalogInV2Voltage": {
                int value = this.getVoltage();
                if (true)
                    updateStateFn.accept(channel, transformBrickletAnalogInV2VoltageGetter0(value, cfg));
            }
                return;
            case "BrickletAnalogInV2AnalogValue": {
                int value = this.getAnalogValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletAnalogInV2AnalogValueGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletAnalogInV2VoltageCallback0(int voltage,
            BrickletAnalogInV2Config cfg) {
        return new QuantityType<>(voltage / 1000.0, SmartHomeUnits.VOLT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletAnalogInV2AnalogValueCallback0(int value,
            BrickletAnalogInV2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletAnalogInV2VoltageGetter0(int value,
            BrickletAnalogInV2Config cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.VOLT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletAnalogInV2AnalogValueGetter0(int value,
            BrickletAnalogInV2Config cfg) {
        return new DecimalType(value);
    }

}
