/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.eclipse.smarthome.core.types.StateOption;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletAnalogOut;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Analog Out Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletAnalogOutWrapper extends BrickletAnalogOut implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletAnalogOut.DEVICE_DISPLAY_NAME,
            "brickletanalogout", BrickletAnalogOut.DEVICE_IDENTIFIER, BrickletAnalogOutWrapper.class,
            BrickletAnalogOutActions.class, "2.0.0", false);

    public BrickletAnalogOutWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletAnalogOutWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);

    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletAnalogOutVoltage");
        result.add("BrickletAnalogOutMode");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletAnalogOutVoltage":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletAnalogOutVoltage"), "Voltage",
                                "Number:ElectricPotential")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletAnalogOutVoltage"))
                        .withDescription(
                                "The output voltage. The possible range is 0V to 5V. Sending a command to this channel will set the Mode to Analog Value.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(5)).withStep(BigDecimal.valueOf(0.001))
                                        .withPattern("%.3f %unit%").withReadOnly(false).build().toStateDescription())
                        .build();
            case "BrickletAnalogOutMode":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletAnalogOutMode"), "Mode",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletAnalogOutMode"))
                        .withDescription(
                                "The mode of the output. Setting the mode to Analog Value will result in an output voltage of 0. You can jump to a higher output voltage directly by sending a command to the Voltage Channel.")
                        .withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d")
                                .withReadOnly(false)
                                .withOptions(Arrays.asList(new StateOption("0", "Analog Value"),
                                        new StateOption("1", "1k To Ground"), new StateOption("2", "100k To Ground"),
                                        new StateOption("3", "500k To Ground")))
                                .build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Analog Out Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Generates configurable DC voltage between 0V and 5V")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletAnalogOutVoltage",
                                new ChannelTypeUID("tinkerforge", "BrickletAnalogOutVoltage")).withLabel("Voltage")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletAnalogOutMode",
                                new ChannelTypeUID("tinkerforge", "BrickletAnalogOutMode")).withLabel("Mode").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletanalogout":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletAnalogOutVoltage":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletAnalogOutMode":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletAnalogOutConfig cfg = (BrickletAnalogOutConfig) config.as(BrickletAnalogOutConfig.class);
        switch (channel) {
            case "BrickletAnalogOutVoltage": {
                int value = this.getVoltage();
                if (true)
                    updateStateFn.accept(channel, transformBrickletAnalogOutVoltageGetter0(value, cfg));
            }
                return;
            case "BrickletAnalogOutMode": {
                short value = this.getMode();
                if (true)
                    updateStateFn.accept(channel, transformBrickletAnalogOutModeGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = new ArrayList<SetterRefresh>();

        switch (channel) {
            case "BrickletAnalogOutVoltage":
                if (command instanceof Number) {
                    Number cmd = (Number) command;
                    this.setVoltage((int) Math.round(cmd.doubleValue() * 1000.0));
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of Number.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletAnalogOutMode", 0));
                break;
            case "BrickletAnalogOutMode":
                if (command instanceof Number) {
                    Number cmd = (Number) command;
                    this.setMode(cmd.shortValue());
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of Number.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletAnalogOutVoltage", 0));
                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletAnalogOutVoltageGetter0(int value,
            BrickletAnalogOutConfig cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.VOLT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletAnalogOutModeGetter0(short value,
            BrickletAnalogOutConfig cfg) {
        return new DecimalType(value);
    }

}
