/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletCurrent12;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Current12 Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletCurrent12Wrapper extends BrickletCurrent12 implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletCurrent12.DEVICE_DISPLAY_NAME,
            "brickletcurrent12", BrickletCurrent12.DEVICE_IDENTIFIER, BrickletCurrent12Wrapper.class,
            BrickletCurrent12Actions.class, "2.0.0", false);

    public BrickletCurrent12Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletCurrent12Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletCurrent12Config cfg = (BrickletCurrent12Config) config.as(BrickletCurrent12Config.class);
        this.addCurrentListener(this.reg((current) -> {
            if (true) {
                updateStateFn.accept("BrickletCurrent12Current",
                        transformBrickletCurrent12CurrentCallback0(current, cfg));
            }
        }, this::removeCurrentListener));
        this.addOverCurrentListener(this.reg(() -> {
            if (true) {
                updateStateFn.accept("BrickletCurrent12OverCurrent",
                        transformBrickletCurrent12OverCurrentCallback0(cfg));
            }
        }, this::removeOverCurrentListener));
        this.addAnalogValueListener(this.reg((value) -> {
            if (true) {
                updateStateFn.accept("BrickletCurrent12AnalogValue",
                        transformBrickletCurrent12AnalogValueCallback0(value, cfg));
            }
        }, this::removeAnalogValueListener));

        {
            BrickletCurrent12CurrentConfig channelCfg = getChannelConfigFn.apply("BrickletCurrent12Current")
                    .as(BrickletCurrent12CurrentConfig.class);
            this.setCurrentCallbackPeriod(channelCfg.updateInterval);
            this.setCurrentCallbackThreshold('x', (short) 0, (short) 0);
        }
        {
            BrickletCurrent12AnalogValueConfig channelCfg = getChannelConfigFn.apply("BrickletCurrent12AnalogValue")
                    .as(BrickletCurrent12AnalogValueConfig.class);
            this.setAnalogValueCallbackPeriod(channelCfg.updateInterval);
            this.setAnalogValueCallbackThreshold('x', (short) 0, (short) 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setCurrentCallbackPeriod(0);
        this.setAnalogValueCallbackPeriod(0);

    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletCurrent12Current");
        result.add("BrickletCurrent12OverCurrent");
        result.add("BrickletCurrent12AnalogValue");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletCurrent12Current":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletCurrent12Current"), "Current",
                                "Number:ElectricCurrent")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletCurrent12Current"))
                        .withDescription("The current of the sensor.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-12.5))
                                        .withMaximum(BigDecimal.valueOf(12.5)).withStep(BigDecimal.valueOf(0.001))
                                        .withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletCurrent12OverCurrent":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletCurrent12OverCurrent"), "Over Current",
                                "Switch")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletCurrent12OverCurrent"))
                        .withDescription(
                                "Enabled if more than 12.5A were measured. To reset this value you have to power cycle the Bricklet.")
                        .build();
            case "BrickletCurrent12AnalogValue":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletCurrent12AnalogValue"), "Analog Value",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletCurrent12AnalogValue"))
                        .withDescription(
                                "<br/>Returns the value as read by a 12-bit analog-to-digital converter.<br/><br/>.. note::<br/> The value returned by :func:`Get Current` is averaged over several samples<br/> to yield less noise, while :func:`Get Analog Value` gives back raw<br/> unfiltered analog values. The only reason to use :func:`Get Analog Value` is,<br/> if you need the full resolution of the analog-to-digital converter.<br/>")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(4095)).withStep(BigDecimal.valueOf(1))
                                        .withPattern("%d").withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Current12 Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Measures AC/DC current between -12.5A and +12.5A")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletCurrent12Current",
                                new ChannelTypeUID("tinkerforge", "BrickletCurrent12Current")).withLabel("Current")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletCurrent12OverCurrent",
                                new ChannelTypeUID("tinkerforge", "BrickletCurrent12OverCurrent"))
                                        .withLabel("Over Current").build(),
                        new ChannelDefinitionBuilder("BrickletCurrent12AnalogValue",
                                new ChannelTypeUID("tinkerforge", "BrickletCurrent12AnalogValue"))
                                        .withLabel("Analog Value").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletcurrent12":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletCurrent12Current":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletCurrent12OverCurrent":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletCurrent12AnalogValue":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletCurrent12Config cfg = (BrickletCurrent12Config) config.as(BrickletCurrent12Config.class);
        switch (channel) {
            case "BrickletCurrent12Current": {
                short value = this.getCurrent();
                if (true)
                    updateStateFn.accept(channel, transformBrickletCurrent12CurrentGetter0(value, cfg));
            }
                return;
            case "BrickletCurrent12OverCurrent": {
                boolean value = this.isOverCurrent();
                if (true)
                    updateStateFn.accept(channel, transformBrickletCurrent12OverCurrentGetter0(value, cfg));
            }
                return;
            case "BrickletCurrent12AnalogValue": {
                int value = this.getAnalogValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletCurrent12AnalogValueGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCurrent12CurrentCallback0(short current,
            BrickletCurrent12Config cfg) {
        return new QuantityType<>(current / 1000.0, SmartHomeUnits.AMPERE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCurrent12OverCurrentCallback0(
            BrickletCurrent12Config cfg) {
        return OnOffType.ON;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCurrent12AnalogValueCallback0(int value,
            BrickletCurrent12Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCurrent12CurrentGetter0(short value,
            BrickletCurrent12Config cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.AMPERE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCurrent12OverCurrentGetter0(boolean value,
            BrickletCurrent12Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCurrent12AnalogValueGetter0(int value,
            BrickletCurrent12Config cfg) {
        return new DecimalType(value);
    }

}
