/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletDistanceIR;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Distance IR Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDistanceIRActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Distance")
    public @ActionOutput(name = "distance", type = "int") Map<String, Object> brickletDistanceIRGetDistance()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRWrapper dev = ((@Nullable BrickletDistanceIRWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getDistance();

        result.put("distance", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRGetDistance(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRActions) {
            return ((BrickletDistanceIRActions) actions).brickletDistanceIRGetDistance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRActions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "value", type = "int") Map<String, Object> brickletDistanceIRGetAnalogValue()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRWrapper dev = ((@Nullable BrickletDistanceIRWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getAnalogValue();

        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRGetAnalogValue(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRActions) {
            return ((BrickletDistanceIRActions) actions).brickletDistanceIRGetAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRActions class.");
        }
    }

    @RuleAction(label = "Get Sampling Point")
    public @ActionOutput(name = "distance", type = "int") Map<String, Object> brickletDistanceIRGetSamplingPoint(
            @ActionInput(name = "position") short position) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRWrapper dev = ((@Nullable BrickletDistanceIRWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getSamplingPoint(position);

        result.put("distance", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRGetSamplingPoint(@Nullable ThingActions actions, short position)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRActions) {
            return ((BrickletDistanceIRActions) actions).brickletDistanceIRGetSamplingPoint(position);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletDistanceIRGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRWrapper dev = ((@Nullable BrickletDistanceIRWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletDistanceIR.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRActions) {
            return ((BrickletDistanceIRActions) actions).brickletDistanceIRGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRActions class.");
        }
    }
}
