/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletDistanceIRV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Distance IR Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDistanceIRV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Distance")
    public @ActionOutput(name = "distance", type = "int") Map<String, Object> brickletDistanceIRV2GetDistance()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRV2Wrapper dev = ((@Nullable BrickletDistanceIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getDistance();

        result.put("distance", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetDistance(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetDistance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "analogValue", type = "long") Map<String, Object> brickletDistanceIRV2GetAnalogValue()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRV2Wrapper dev = ((@Nullable BrickletDistanceIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.getAnalogValue();

        result.put("analogValue", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetAnalogValue(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Moving Average Configuration")
    public @ActionOutput(name = "movingAverageLength", type = "int") Map<String, Object> brickletDistanceIRV2GetMovingAverageConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRV2Wrapper dev = ((@Nullable BrickletDistanceIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getMovingAverageConfiguration();

        result.put("movingAverageLength", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetMovingAverageConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetMovingAverageConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Distance LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletDistanceIRV2GetDistanceLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRV2Wrapper dev = ((@Nullable BrickletDistanceIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getDistanceLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetDistanceLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetDistanceLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Sensor Type")
    public @ActionOutput(name = "sensor", type = "int") Map<String, Object> brickletDistanceIRV2GetSensorType()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRV2Wrapper dev = ((@Nullable BrickletDistanceIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getSensorType();

        result.put("sensor", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetSensorType(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetSensorType();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletDistanceIRV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRV2Wrapper dev = ((@Nullable BrickletDistanceIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletDistanceIRV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRV2Wrapper dev = ((@Nullable BrickletDistanceIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletDistanceIRV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRV2Wrapper dev = ((@Nullable BrickletDistanceIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletDistanceIRV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRV2Wrapper dev = ((@Nullable BrickletDistanceIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletDistanceIRV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "int[]") @ActionOutput(name = "firmwareVersion", type = "int[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletDistanceIRV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDistanceIRV2Wrapper dev = ((@Nullable BrickletDistanceIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletDistanceIRV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }
}
