/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletIndustrialAnalogOutV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Industrial Analog Out Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialAnalogOutV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set Enabled")
    public void brickletIndustrialAnalogOutV2SetEnabled(@ActionInput(name = "enabled") boolean enabled)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletIndustrialAnalogOutV2Wrapper dev = ((@Nullable BrickletIndustrialAnalogOutV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setEnabled(enabled);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialAnalogOutV2Enabled")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialAnalogOutV2Enabled"), RefreshType.REFRESH);
        }
    }

    public static void brickletIndustrialAnalogOutV2SetEnabled(@Nullable ThingActions actions, boolean enabled)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutV2Actions) {
            ((BrickletIndustrialAnalogOutV2Actions) actions).brickletIndustrialAnalogOutV2SetEnabled(enabled);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Enabled")
    public @ActionOutput(name = "enabled", type = "boolean") Map<String, Object> brickletIndustrialAnalogOutV2GetEnabled()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutV2Wrapper dev = ((@Nullable BrickletIndustrialAnalogOutV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.getEnabled();

        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutV2GetEnabled(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutV2Actions) {
            return ((BrickletIndustrialAnalogOutV2Actions) actions).brickletIndustrialAnalogOutV2GetEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickletIndustrialAnalogOutV2GetVoltage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutV2Wrapper dev = ((@Nullable BrickletIndustrialAnalogOutV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getVoltage();

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutV2GetVoltage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutV2Actions) {
            return ((BrickletIndustrialAnalogOutV2Actions) actions).brickletIndustrialAnalogOutV2GetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Current")
    public @ActionOutput(name = "current", type = "int") Map<String, Object> brickletIndustrialAnalogOutV2GetCurrent()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutV2Wrapper dev = ((@Nullable BrickletIndustrialAnalogOutV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getCurrent();

        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutV2GetCurrent(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutV2Actions) {
            return ((BrickletIndustrialAnalogOutV2Actions) actions).brickletIndustrialAnalogOutV2GetCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "voltageRange", type = "int") @ActionOutput(name = "currentRange", type = "int") Map<String, Object> brickletIndustrialAnalogOutV2GetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutV2Wrapper dev = ((@Nullable BrickletIndustrialAnalogOutV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletIndustrialAnalogOutV2.Configuration value = dev.getConfiguration();

        result.put("voltageRange", value.voltageRange);
        result.put("currentRange", value.currentRange);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutV2GetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutV2Actions) {
            return ((BrickletIndustrialAnalogOutV2Actions) actions).brickletIndustrialAnalogOutV2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Out LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletIndustrialAnalogOutV2GetOutLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutV2Wrapper dev = ((@Nullable BrickletIndustrialAnalogOutV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getOutLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutV2GetOutLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutV2Actions) {
            return ((BrickletIndustrialAnalogOutV2Actions) actions).brickletIndustrialAnalogOutV2GetOutLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Out LED Status Config")
    public @ActionOutput(name = "min", type = "int") @ActionOutput(name = "max", type = "int") @ActionOutput(name = "config", type = "int") Map<String, Object> brickletIndustrialAnalogOutV2GetOutLEDStatusConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutV2Wrapper dev = ((@Nullable BrickletIndustrialAnalogOutV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletIndustrialAnalogOutV2.OutLEDStatusConfig value = dev.getOutLEDStatusConfig();

        result.put("min", value.min);
        result.put("max", value.max);
        result.put("config", value.config);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutV2GetOutLEDStatusConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutV2Actions) {
            return ((BrickletIndustrialAnalogOutV2Actions) actions)
                    .brickletIndustrialAnalogOutV2GetOutLEDStatusConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletIndustrialAnalogOutV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutV2Wrapper dev = ((@Nullable BrickletIndustrialAnalogOutV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutV2Actions) {
            return ((BrickletIndustrialAnalogOutV2Actions) actions).brickletIndustrialAnalogOutV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletIndustrialAnalogOutV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutV2Wrapper dev = ((@Nullable BrickletIndustrialAnalogOutV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutV2Actions) {
            return ((BrickletIndustrialAnalogOutV2Actions) actions).brickletIndustrialAnalogOutV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletIndustrialAnalogOutV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutV2Wrapper dev = ((@Nullable BrickletIndustrialAnalogOutV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutV2Actions) {
            return ((BrickletIndustrialAnalogOutV2Actions) actions).brickletIndustrialAnalogOutV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletIndustrialAnalogOutV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutV2Wrapper dev = ((@Nullable BrickletIndustrialAnalogOutV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletIndustrialAnalogOutV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutV2Actions) {
            return ((BrickletIndustrialAnalogOutV2Actions) actions).brickletIndustrialAnalogOutV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "int[]") @ActionOutput(name = "firmwareVersion", type = "int[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletIndustrialAnalogOutV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutV2Wrapper dev = ((@Nullable BrickletIndustrialAnalogOutV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletIndustrialAnalogOutV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutV2Actions) {
            return ((BrickletIndustrialAnalogOutV2Actions) actions).brickletIndustrialAnalogOutV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutV2Actions class.");
        }
    }
}
