/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletIndustrialAnalogOutV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Industrial Analog Out Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletIndustrialAnalogOutV2Wrapper extends BrickletIndustrialAnalogOutV2
        implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletIndustrialAnalogOutV2.DEVICE_DISPLAY_NAME,
            "brickletindustrialanalogoutv2", BrickletIndustrialAnalogOutV2.DEVICE_IDENTIFIER,
            BrickletIndustrialAnalogOutV2Wrapper.class, BrickletIndustrialAnalogOutV2Actions.class, "2.0.0", true);

    public BrickletIndustrialAnalogOutV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletIndustrialAnalogOutV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletIndustrialAnalogOutV2Config cfg = (BrickletIndustrialAnalogOutV2Config) config
                .as(BrickletIndustrialAnalogOutV2Config.class);

        this.setConfiguration(cfg.voltageRange, cfg.currentRange);
        this.setOutLEDConfig(cfg.outLEDConfig);
        this.setOutLEDStatusConfig(
                (int) (cfg.outLEDStatusMinimum.doubleValue() * (cfg.controlVoltage == 1 ? 1000.0 : 1000000.0)),
                (int) (cfg.outLEDStatusMaximum.doubleValue() * (cfg.controlVoltage == 1 ? 1000.0 : 1000000.0)),
                cfg.outLEDStatusMode);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        BrickletIndustrialAnalogOutV2Config cfg = (BrickletIndustrialAnalogOutV2Config) config
                .as(BrickletIndustrialAnalogOutV2Config.class);
        List<String> result = new ArrayList<String>();
        result.add("BrickletIndustrialAnalogOutV2Enabled");
        if (cfg.controlVoltage == 0) {
            result.add("BrickletIndustrialAnalogOutV2Current");
        }
        if (cfg.controlVoltage == 1) {
            result.add("BrickletIndustrialAnalogOutV2Voltage");
        }
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletIndustrialAnalogOutV2Enabled":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialAnalogOutV2Enabled"),
                                "Output Enabled", "Switch")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletIndustrialAnalogOutV2Enabled"))
                        .withDescription("Enables/disables the output of voltage and current.").build();
            case "BrickletIndustrialAnalogOutV2Voltage":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialAnalogOutV2Voltage"),
                                "Output Voltage", "Number:ElectricPotential")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletIndustrialAnalogOutV2Voltage"))
                        .withDescription(
                                "The output voltage in V. The output voltage and output current are linked. Changing the output voltage also changes the output current.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(10)).withStep(BigDecimal.valueOf(0.001))
                                        .withPattern("%.3f %unit%").withReadOnly(false).build().toStateDescription())
                        .build();
            case "BrickletIndustrialAnalogOutV2Current":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialAnalogOutV2Current"),
                                "Output Current", "Number:ElectricCurrent")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletIndustrialAnalogOutV2Current"))
                        .withDescription(
                                "The output current in A. The output current and output voltage are linked. Changing the output current also changes the output voltage.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(0.024)).withStep(BigDecimal.valueOf(1e-06))
                                        .withPattern("%.6f %unit%").withReadOnly(false).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Industrial Analog Out Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Generates configurable DC voltage and current, 0V to 10V and 4mA to 20mA")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletIndustrialAnalogOutV2Enabled",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialAnalogOutV2Enabled"))
                                        .withLabel("Output Enabled").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialAnalogOutV2Current",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialAnalogOutV2Current"))
                                        .withLabel("Output Current").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialAnalogOutV2Voltage",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialAnalogOutV2Voltage"))
                                        .withLabel("Output Voltage").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletindustrialanalogoutv2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder
                                .create("controlVoltage", Type.INTEGER).withDefault("1")
                                .withDescription(
                                        "Sets the output configuration. As the output voltage and current level depend on each other, only one can be controlled at the same time.")
                                .withLabel("Output Configuration").withLimitToOptions(true)
                                .withOptions(
                                        Arrays.asList(new ParameterOption("0", "Current"),
                                                new ParameterOption("1", "Voltage")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("voltageRange", Type.INTEGER).withDefault("1")
                                .withDescription(
                                        "Configures the voltage range. The resolution will always be 12 bit. This means, that the precision is higher with a smaller range.")
                                .withLabel("Voltage Range").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "0 To 5V"),
                                        new ParameterOption("1", "0 To 10V")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("currentRange", Type.INTEGER).withDefault("0")
                                .withDescription(
                                        "Configures the current range. The resolution will always be 12 bit. This means, that the precision is higher with a smaller range.")
                                .withLabel("Current Range").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "4 To 20mA"),
                                        new ParameterOption("1", "0 To 20mA"), new ParameterOption("2", "0 To 24mA")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("outLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "You can turn the Out LED off, on or show a heartbeat. You can also set the LED to Out Status. In this mode the LED can either be turned on with a pre-defined threshold or the intensity of the LED can change with the output value (voltage or current).")
                                .withLabel("Output LED Configuration").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Out Status")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("outLEDStatusMode", Type.INTEGER).withDefault("1")
                                .withDescription(
                                        "For each channel you can choose between threshold and intensity mode.<br/><br/>In threshold mode you can define a positive or a negative threshold.<br/>For a positive threshold set the \"min\" parameter to the threshold value in mV or<br/>µA above which the LED should turn on and set the \"max\" parameter to 0. Example:<br/>If you set a positive threshold of 5V, the LED will turn on as soon as the<br/>output value exceeds 5V and turn off again if it goes below 5V.<br/>For a negative threshold set the \"max\" parameter to the threshold value in mV or<br/>µA below which the LED should turn on and set the \"min\" parameter to 0. Example:<br/>If you set a negative threshold of 5V, the LED will turn on as soon as the<br/>output value goes below 5V and the LED will turn off when the output value<br/>exceeds 5V.<br/><br/>In intensity mode you can define a range mV or µA that is used to scale the brightness<br/>of the LED. Example with min=2V, max=8V: The LED is off at 2V and below, on at<br/>8V and above and the brightness is linearly scaled between the values 2V and 8V.<br/>If the min value is greater than the max value, the LED brightness is scaled the<br/>other way around.")
                                .withLabel("Output LED Status Mode").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Threshold"),
                                        new ParameterOption("1", "Intensity")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("outLEDStatusMinimum", Type.DECIMAL).withDefault("0")
                                .withDescription("See LED Status Mode for further explaination.")
                                .withLabel("Output LED Status Maximum").withMinimum(BigDecimal.valueOf(0))
                                .withMaximum(BigDecimal.valueOf(10)).build(),
                        ConfigDescriptionParameterBuilder.create("outLEDStatusMaximum", Type.DECIMAL).withDefault("10")
                                .withDescription("See LED Status Mode for further explaination.")
                                .withLabel("Output LED Status Maximum").withMinimum(BigDecimal.valueOf(0))
                                .withMaximum(BigDecimal.valueOf(10)).build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialAnalogOutV2Enabled":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialAnalogOutV2Voltage":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialAnalogOutV2Current":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletIndustrialAnalogOutV2Config cfg = (BrickletIndustrialAnalogOutV2Config) config
                .as(BrickletIndustrialAnalogOutV2Config.class);
        switch (channel) {
            case "BrickletIndustrialAnalogOutV2Enabled": {
                boolean value = this.getEnabled();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialAnalogOutV2EnabledGetter0(value, cfg));
            }
                return;
            case "BrickletIndustrialAnalogOutV2Current": {
                int value = this.getCurrent();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialAnalogOutV2CurrentGetter0(value, cfg));
            }
                return;
            case "BrickletIndustrialAnalogOutV2Voltage": {
                int value = this.getVoltage();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialAnalogOutV2VoltageGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            case "BrickletIndustrialAnalogOutV2Enabled":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setEnabled(cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialAnalogOutV2Current":
                if (command instanceof Number) {
                    Number cmd = (Number) command;
                    this.setCurrent((int) (cmd.doubleValue() * 1000000.0));
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of Number.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialAnalogOutV2Voltage":
                if (command instanceof Number) {
                    Number cmd = (Number) command;
                    this.setVoltage((int) (cmd.doubleValue() * 1000.0));
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of Number.",
                            command.getClass().getName(), channel);
                }

                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialAnalogOutV2EnabledGetter0(boolean value,
            BrickletIndustrialAnalogOutV2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialAnalogOutV2CurrentGetter0(int value,
            BrickletIndustrialAnalogOutV2Config cfg) {
        return new QuantityType<>(value / 1000000.0, SmartHomeUnits.AMPERE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialAnalogOutV2VoltageGetter0(int value,
            BrickletIndustrialAnalogOutV2Config cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.VOLT);
    }

}
