/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletLEDStrip;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the LED Strip Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLEDStripActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set RGB Values")
    public void brickletLEDStripSetRGBValues(@ActionInput(name = "index") int index,
            @ActionInput(name = "length") short length, @ActionInput(name = "r") short[] r,
            @ActionInput(name = "g") short[] g, @ActionInput(name = "b") short[] b) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setRGBValues(index, length, r, g, b);

    }

    public static void brickletLEDStripSetRGBValues(@Nullable ThingActions actions, int index, short length, short[] r,
            short[] g, short[] b) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            ((BrickletLEDStripActions) actions).brickletLEDStripSetRGBValues(index, length, r, g, b);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get RGB Values")
    public @ActionOutput(name = "r", type = "short[]") @ActionOutput(name = "g", type = "short[]") @ActionOutput(name = "b", type = "short[]") Map<String, Object> brickletLEDStripGetRGBValues(
            @ActionInput(name = "index") int index, @ActionInput(name = "length") short length)
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLEDStrip.RGBValues value = dev.getRGBValues(index, length);

        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetRGBValues(@Nullable ThingActions actions, int index,
            short length) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetRGBValues(index, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Frame Duration")
    public @ActionOutput(name = "duration", type = "int") Map<String, Object> brickletLEDStripGetFrameDuration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getFrameDuration();

        result.put("duration", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetFrameDuration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetFrameDuration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Supply Voltage")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickletLEDStripGetSupplyVoltage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getSupplyVoltage();

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetSupplyVoltage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetSupplyVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Clock Frequency")
    public @ActionOutput(name = "frequency", type = "long") Map<String, Object> brickletLEDStripGetClockFrequency()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.getClockFrequency();

        result.put("frequency", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetClockFrequency(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetClockFrequency();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Chip Type")
    public @ActionOutput(name = "chip", type = "int") Map<String, Object> brickletLEDStripGetChipType()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipType();

        result.put("chip", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetChipType(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetChipType();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Set RGBW Values")
    public void brickletLEDStripSetRGBWValues(@ActionInput(name = "index") int index,
            @ActionInput(name = "length") short length, @ActionInput(name = "r") short[] r,
            @ActionInput(name = "g") short[] g, @ActionInput(name = "b") short[] b, @ActionInput(name = "w") short[] w)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setRGBWValues(index, length, r, g, b, w);

    }

    public static void brickletLEDStripSetRGBWValues(@Nullable ThingActions actions, int index, short length, short[] r,
            short[] g, short[] b, short[] w) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            ((BrickletLEDStripActions) actions).brickletLEDStripSetRGBWValues(index, length, r, g, b, w);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get RGBW Values")
    public @ActionOutput(name = "r", type = "short[]") @ActionOutput(name = "g", type = "short[]") @ActionOutput(name = "b", type = "short[]") @ActionOutput(name = "w", type = "short[]") Map<String, Object> brickletLEDStripGetRGBWValues(
            @ActionInput(name = "index") int index, @ActionInput(name = "length") short length)
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLEDStrip.RGBWValues value = dev.getRGBWValues(index, length);

        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        result.put("w", value.w);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetRGBWValues(@Nullable ThingActions actions, int index,
            short length) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetRGBWValues(index, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Channel Mapping")
    public @ActionOutput(name = "mapping", type = "short") Map<String, Object> brickletLEDStripGetChannelMapping()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getChannelMapping();

        result.put("mapping", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetChannelMapping(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetChannelMapping();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletLEDStripGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLEDStripWrapper dev = ((@Nullable BrickletLEDStripWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLEDStrip.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }
}
