/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletLaserRangeFinder;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Laser Range Finder Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLaserRangeFinderActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Distance")
    public @ActionOutput(name = "distance", type = "int") Map<String, Object> brickletLaserRangeFinderGetDistance()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderWrapper dev = ((@Nullable BrickletLaserRangeFinderWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getDistance();

        result.put("distance", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderGetDistance(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderActions) {
            return ((BrickletLaserRangeFinderActions) actions).brickletLaserRangeFinderGetDistance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderActions class.");
        }
    }

    @RuleAction(label = "Get Velocity")
    public @ActionOutput(name = "velocity", type = "short") Map<String, Object> brickletLaserRangeFinderGetVelocity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderWrapper dev = ((@Nullable BrickletLaserRangeFinderWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getVelocity();

        result.put("velocity", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderGetVelocity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderActions) {
            return ((BrickletLaserRangeFinderActions) actions).brickletLaserRangeFinderGetVelocity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderActions class.");
        }
    }

    @RuleAction(label = "Get Mode")
    public @ActionOutput(name = "mode", type = "short") Map<String, Object> brickletLaserRangeFinderGetMode()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderWrapper dev = ((@Nullable BrickletLaserRangeFinderWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getMode();

        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderGetMode(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderActions) {
            return ((BrickletLaserRangeFinderActions) actions).brickletLaserRangeFinderGetMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderActions class.");
        }
    }

    @RuleAction(label = "Enable Laser")
    public void brickletLaserRangeFinderEnableLaser() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletLaserRangeFinderWrapper dev = ((@Nullable BrickletLaserRangeFinderWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.enableLaser();
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletLaserRangeFinderEnableLaser")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletLaserRangeFinderEnableLaser"), RefreshType.REFRESH);
        }
    }

    public static void brickletLaserRangeFinderEnableLaser(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderActions) {
            ((BrickletLaserRangeFinderActions) actions).brickletLaserRangeFinderEnableLaser();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderActions class.");
        }
    }

    @RuleAction(label = "Disable Laser")
    public void brickletLaserRangeFinderDisableLaser() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletLaserRangeFinderWrapper dev = ((@Nullable BrickletLaserRangeFinderWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.disableLaser();
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletLaserRangeFinderEnableLaser")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletLaserRangeFinderEnableLaser"), RefreshType.REFRESH);
        }
    }

    public static void brickletLaserRangeFinderDisableLaser(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderActions) {
            ((BrickletLaserRangeFinderActions) actions).brickletLaserRangeFinderDisableLaser();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderActions class.");
        }
    }

    @RuleAction(label = "Is Laser Enabled")
    public @ActionOutput(name = "laserEnabled", type = "boolean") Map<String, Object> brickletLaserRangeFinderIsLaserEnabled()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderWrapper dev = ((@Nullable BrickletLaserRangeFinderWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.isLaserEnabled();

        result.put("laserEnabled", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderIsLaserEnabled(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderActions) {
            return ((BrickletLaserRangeFinderActions) actions).brickletLaserRangeFinderIsLaserEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "acquisitionCount", type = "short") @ActionOutput(name = "enableQuickTermination", type = "boolean") @ActionOutput(name = "thresholdValue", type = "short") @ActionOutput(name = "measurementFrequency", type = "int") Map<String, Object> brickletLaserRangeFinderGetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderWrapper dev = ((@Nullable BrickletLaserRangeFinderWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLaserRangeFinder.Configuration value = dev.getConfiguration();

        result.put("acquisitionCount", value.acquisitionCount);
        result.put("enableQuickTermination", value.enableQuickTermination);
        result.put("thresholdValue", value.thresholdValue);
        result.put("measurementFrequency", value.measurementFrequency);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderGetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderActions) {
            return ((BrickletLaserRangeFinderActions) actions).brickletLaserRangeFinderGetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderActions class.");
        }
    }

    @RuleAction(label = "Get Moving Average")
    public @ActionOutput(name = "distanceAverageLength", type = "short") @ActionOutput(name = "velocityAverageLength", type = "short") Map<String, Object> brickletLaserRangeFinderGetMovingAverage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderWrapper dev = ((@Nullable BrickletLaserRangeFinderWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLaserRangeFinder.MovingAverage value = dev.getMovingAverage();

        result.put("distanceAverageLength", value.distanceAverageLength);
        result.put("velocityAverageLength", value.velocityAverageLength);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderGetMovingAverage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderActions) {
            return ((BrickletLaserRangeFinderActions) actions).brickletLaserRangeFinderGetMovingAverage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderActions class.");
        }
    }

    @RuleAction(label = "Get Sensor Hardware Version")
    public @ActionOutput(name = "version", type = "short") Map<String, Object> brickletLaserRangeFinderGetSensorHardwareVersion()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderWrapper dev = ((@Nullable BrickletLaserRangeFinderWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getSensorHardwareVersion();

        result.put("version", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderGetSensorHardwareVersion(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderActions) {
            return ((BrickletLaserRangeFinderActions) actions).brickletLaserRangeFinderGetSensorHardwareVersion();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletLaserRangeFinderGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderWrapper dev = ((@Nullable BrickletLaserRangeFinderWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLaserRangeFinder.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderActions) {
            return ((BrickletLaserRangeFinderActions) actions).brickletLaserRangeFinderGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderActions class.");
        }
    }
}
