/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletLaserRangeFinderV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Laser Range Finder Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLaserRangeFinderV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Distance")
    public @ActionOutput(name = "distance", type = "int") Map<String, Object> brickletLaserRangeFinderV2GetDistance()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getDistance();

        result.put("distance", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderV2GetDistance(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            return ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2GetDistance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Velocity")
    public @ActionOutput(name = "velocity", type = "int") Map<String, Object> brickletLaserRangeFinderV2GetVelocity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getVelocity();

        result.put("velocity", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderV2GetVelocity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            return ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2GetVelocity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }

    @RuleAction(label = "Set Enable")
    public void brickletLaserRangeFinderV2SetEnable(@ActionInput(name = "enable") boolean enable)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setEnable(enable);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletLaserRangeFinderV2EnableLaser")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletLaserRangeFinderV2EnableLaser"), RefreshType.REFRESH);
        }
    }

    public static void brickletLaserRangeFinderV2SetEnable(@Nullable ThingActions actions, boolean enable)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2SetEnable(enable);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Enable")
    public @ActionOutput(name = "enable", type = "boolean") Map<String, Object> brickletLaserRangeFinderV2GetEnable()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.getEnable();

        result.put("enable", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderV2GetEnable(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            return ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2GetEnable();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "acquisitionCount", type = "int") @ActionOutput(name = "enableQuickTermination", type = "boolean") @ActionOutput(name = "thresholdValue", type = "int") @ActionOutput(name = "measurementFrequency", type = "int") Map<String, Object> brickletLaserRangeFinderV2GetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLaserRangeFinderV2.Configuration value = dev.getConfiguration();

        result.put("acquisitionCount", value.acquisitionCount);
        result.put("enableQuickTermination", value.enableQuickTermination);
        result.put("thresholdValue", value.thresholdValue);
        result.put("measurementFrequency", value.measurementFrequency);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderV2GetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            return ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Distance LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletLaserRangeFinderV2GetDistanceLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getDistanceLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderV2GetDistanceLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            return ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2GetDistanceLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Moving Average")
    public @ActionOutput(name = "distanceAverageLength", type = "int") @ActionOutput(name = "velocityAverageLength", type = "int") Map<String, Object> brickletLaserRangeFinderV2GetMovingAverage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLaserRangeFinderV2.MovingAverage value = dev.getMovingAverage();

        result.put("distanceAverageLength", value.distanceAverageLength);
        result.put("velocityAverageLength", value.velocityAverageLength);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderV2GetMovingAverage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            return ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2GetMovingAverage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Offset Calibration")
    public @ActionOutput(name = "offset", type = "int") Map<String, Object> brickletLaserRangeFinderV2GetOffsetCalibration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getOffsetCalibration();

        result.put("offset", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderV2GetOffsetCalibration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            return ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2GetOffsetCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletLaserRangeFinderV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            return ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletLaserRangeFinderV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            return ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletLaserRangeFinderV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            return ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletLaserRangeFinderV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLaserRangeFinderV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            return ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "int[]") @ActionOutput(name = "firmwareVersion", type = "int[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletLaserRangeFinderV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLaserRangeFinderV2Wrapper dev = ((@Nullable BrickletLaserRangeFinderV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLaserRangeFinderV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletLaserRangeFinderV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLaserRangeFinderV2Actions) {
            return ((BrickletLaserRangeFinderV2Actions) actions).brickletLaserRangeFinderV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLaserRangeFinderV2Actions class.");
        }
    }
}
