/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletMultiTouch;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Multi Touch Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMultiTouchActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Touch State")
    public @ActionOutput(name = "state", type = "int") Map<String, Object> brickletMultiTouchGetTouchState()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchWrapper dev = ((@Nullable BrickletMultiTouchWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getTouchState();

        result.put("state", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchGetTouchState(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchActions) {
            return ((BrickletMultiTouchActions) actions).brickletMultiTouchGetTouchState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchActions class.");
        }
    }

    @RuleAction(label = "Recalibrate")
    public void brickletMultiTouchRecalibrate() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletMultiTouchWrapper dev = ((@Nullable BrickletMultiTouchWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.recalibrate();

    }

    public static void brickletMultiTouchRecalibrate(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchActions) {
            ((BrickletMultiTouchActions) actions).brickletMultiTouchRecalibrate();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchActions class.");
        }
    }

    @RuleAction(label = "Get Electrode Config")
    public @ActionOutput(name = "enabledElectrodes", type = "int") Map<String, Object> brickletMultiTouchGetElectrodeConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchWrapper dev = ((@Nullable BrickletMultiTouchWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getElectrodeConfig();

        result.put("enabledElectrodes", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchGetElectrodeConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchActions) {
            return ((BrickletMultiTouchActions) actions).brickletMultiTouchGetElectrodeConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchActions class.");
        }
    }

    @RuleAction(label = "Get Electrode Sensitivity")
    public @ActionOutput(name = "sensitivity", type = "short") Map<String, Object> brickletMultiTouchGetElectrodeSensitivity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchWrapper dev = ((@Nullable BrickletMultiTouchWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getElectrodeSensitivity();

        result.put("sensitivity", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchGetElectrodeSensitivity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchActions) {
            return ((BrickletMultiTouchActions) actions).brickletMultiTouchGetElectrodeSensitivity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletMultiTouchGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchWrapper dev = ((@Nullable BrickletMultiTouchWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletMultiTouch.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchActions) {
            return ((BrickletMultiTouchActions) actions).brickletMultiTouchGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchActions class.");
        }
    }
}
