/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletMultiTouchV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Multi Touch Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMultiTouchV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Touch State")
    public @ActionOutput(name = "state", type = "boolean[]") Map<String, Object> brickletMultiTouchV2GetTouchState()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchV2Wrapper dev = ((@Nullable BrickletMultiTouchV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean[] value = dev.getTouchState();

        result.put("state", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetTouchState(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetTouchState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Recalibrate")
    public void brickletMultiTouchV2Recalibrate() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletMultiTouchV2Wrapper dev = ((@Nullable BrickletMultiTouchV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.recalibrate();

    }

    public static void brickletMultiTouchV2Recalibrate(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2Recalibrate();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Electrode Config")
    public @ActionOutput(name = "enabledElectrodes", type = "boolean[]") Map<String, Object> brickletMultiTouchV2GetElectrodeConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchV2Wrapper dev = ((@Nullable BrickletMultiTouchV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean[] value = dev.getElectrodeConfig();

        result.put("enabledElectrodes", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetElectrodeConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetElectrodeConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Electrode Sensitivity")
    public @ActionOutput(name = "sensitivity", type = "int") Map<String, Object> brickletMultiTouchV2GetElectrodeSensitivity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchV2Wrapper dev = ((@Nullable BrickletMultiTouchV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getElectrodeSensitivity();

        result.put("sensitivity", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetElectrodeSensitivity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetElectrodeSensitivity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Touch LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletMultiTouchV2GetTouchLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchV2Wrapper dev = ((@Nullable BrickletMultiTouchV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getTouchLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetTouchLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetTouchLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletMultiTouchV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchV2Wrapper dev = ((@Nullable BrickletMultiTouchV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletMultiTouchV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchV2Wrapper dev = ((@Nullable BrickletMultiTouchV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletMultiTouchV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchV2Wrapper dev = ((@Nullable BrickletMultiTouchV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletMultiTouchV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchV2Wrapper dev = ((@Nullable BrickletMultiTouchV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletMultiTouchV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "int[]") @ActionOutput(name = "firmwareVersion", type = "int[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletMultiTouchV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMultiTouchV2Wrapper dev = ((@Nullable BrickletMultiTouchV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletMultiTouchV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }
}
