/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.StringType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletOutdoorWeather;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Outdoor Weather Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletOutdoorWeatherWrapper extends BrickletOutdoorWeather implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletOutdoorWeather.DEVICE_DISPLAY_NAME,
            "brickletoutdoorweather", BrickletOutdoorWeather.DEVICE_IDENTIFIER, BrickletOutdoorWeatherWrapper.class,
            BrickletOutdoorWeatherActions.class, "2.0.0", true);

    public BrickletOutdoorWeatherWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletOutdoorWeatherWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletOutdoorWeatherConfig cfg = (BrickletOutdoorWeatherConfig) config.as(BrickletOutdoorWeatherConfig.class);

        this.setStationCallbackConfiguration(true);
        this.setSensorCallbackConfiguration(true);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            BrickletOutdoorWeatherSensorIdentifiersConfig channelCfg = getChannelConfigFn
                    .apply("BrickletOutdoorWeatherSensorIdentifiers")
                    .as(BrickletOutdoorWeatherSensorIdentifiersConfig.class);

            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletOutdoorWeatherSensorIdentifiers"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        {
            BrickletOutdoorWeatherStationIdentifiersConfig channelCfg = getChannelConfigFn
                    .apply("BrickletOutdoorWeatherStationIdentifiers")
                    .as(BrickletOutdoorWeatherStationIdentifiersConfig.class);

            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletOutdoorWeatherStationIdentifiers"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setStationCallbackConfiguration(false);
        this.setSensorCallbackConfiguration(false);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletOutdoorWeatherSensorIdentifiers");
        result.add("BrickletOutdoorWeatherStationIdentifiers");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletOutdoorWeatherSensorIdentifiers":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletOutdoorWeatherSensorIdentifiers"), "NOT USED",
                                "String")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletOutdoorWeatherSensorIdentifiers"))
                        .withDescription("NOT USED").build();
            case "BrickletOutdoorWeatherStationIdentifiers":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletOutdoorWeatherStationIdentifiers"),
                                "NOT USED", "String")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletOutdoorWeatherStationIdentifiers"))
                        .withDescription("NOT USED").build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Outdoor Weather Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("433MHz receiver for outdoor weather station")
                .withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder(
                        "BrickletOutdoorWeatherSensorIdentifiers",
                        new ChannelTypeUID("tinkerforge", "BrickletOutdoorWeatherSensorIdentifiers"))
                                .withLabel("Sensor Identifiers")
                                .withDescription(
                                        "The identifiers (number between 0 and 255) of all sensors that have been seen since the startup of the Bricklet.<br/><br/>Each sensor gives itself a random identifier on first startup.<br/><br/>Since firmware version 2.0.2 a sensor is removed from the list if no data was received for 12 hours.")
                                .build(),
                        new ChannelDefinitionBuilder("BrickletOutdoorWeatherStationIdentifiers",
                                new ChannelTypeUID("tinkerforge", "BrickletOutdoorWeatherStationIdentifiers"))
                                        .withLabel("Station Identifiers")
                                        .withDescription(
                                                "The identifiers (number between 0 and 255) of all stations that have been seen since the startup of the Bricklet.<br/><br/>Each station gives itself a random identifier on first startup.<br/><br/>Since firmware version 2.0.2 a station is removed from the list if no data was received for 12 hours.")
                                        .build()))
                .withProperties(props).buildBridge();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletoutdoorweather":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletOutdoorWeatherSensorIdentifiers":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withLabel("Update Interval").withUnit("ms").build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletOutdoorWeatherStationIdentifiers":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withLabel("Update Interval").withUnit("ms").build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletOutdoorWeatherConfig cfg = (BrickletOutdoorWeatherConfig) config.as(BrickletOutdoorWeatherConfig.class);
        switch (channel) {
            case "BrickletOutdoorWeatherSensorIdentifiers": {
                int[] value = this.getSensorIdentifiers();
                if (true)
                    updateStateFn.accept(channel, transformBrickletOutdoorWeatherSensorIdentifiersGetter0(value, cfg));
            }
                return;
            case "BrickletOutdoorWeatherStationIdentifiers": {
                int[] value = this.getStationIdentifiers();
                if (true)
                    updateStateFn.accept(channel, transformBrickletOutdoorWeatherStationIdentifiersGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletOutdoorWeatherSensorIdentifiersGetter0(int[] value,
            BrickletOutdoorWeatherConfig cfg) {
        return new StringType(Arrays.stream(value).mapToObj(String::valueOf).reduce((l, r) -> l + ", " + r).orElse(""));
    }

    private org.eclipse.smarthome.core.types.State transformBrickletOutdoorWeatherStationIdentifiersGetter0(int[] value,
            BrickletOutdoorWeatherConfig cfg) {
        return new StringType(Arrays.stream(value).mapToObj(String::valueOf).reduce((l, r) -> l + ", " + r).orElse(""));
    }

}
