/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletPTCV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the PTC Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletPTCV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletPTCV2GetTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Is Sensor Connected")
    public @ActionOutput(name = "connected", type = "boolean") Map<String, Object> brickletPTCV2IsSensorConnected()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.isSensorConnected();

        result.put("connected", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2IsSensorConnected(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2IsSensorConnected();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Wire Mode")
    public @ActionOutput(name = "mode", type = "int") Map<String, Object> brickletPTCV2GetWireMode()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getWireMode();

        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetWireMode(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetWireMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Resistance")
    public @ActionOutput(name = "resistance", type = "int") Map<String, Object> brickletPTCV2GetResistance()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getResistance();

        result.put("resistance", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetResistance(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetResistance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Noise Rejection Filter")
    public @ActionOutput(name = "filter", type = "int") Map<String, Object> brickletPTCV2GetNoiseRejectionFilter()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getNoiseRejectionFilter();

        result.put("filter", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetNoiseRejectionFilter(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetNoiseRejectionFilter();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Moving Average Configuration")
    public @ActionOutput(name = "movingAverageLengthResistance", type = "int") @ActionOutput(name = "movingAverageLengthTemperature", type = "int") Map<String, Object> brickletPTCV2GetMovingAverageConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletPTCV2.MovingAverageConfiguration value = dev.getMovingAverageConfiguration();

        result.put("movingAverageLengthResistance", value.movingAverageLengthResistance);
        result.put("movingAverageLengthTemperature", value.movingAverageLengthTemperature);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetMovingAverageConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetMovingAverageConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletPTCV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletPTCV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletPTCV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletPTCV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletPTCV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "int[]") @ActionOutput(name = "firmwareVersion", type = "int[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletPTCV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletPTCV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }
}
