/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletPiezoSpeakerV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Piezo Speaker Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletPiezoSpeakerV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set Beep")
    public void brickletPiezoSpeakerV2SetBeep(@ActionInput(name = "frequency") int frequency,
            @ActionInput(name = "volume") int volume, @ActionInput(name = "duration") long duration)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletPiezoSpeakerV2Wrapper dev = ((@Nullable BrickletPiezoSpeakerV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setBeep(frequency, volume, duration);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletPiezoSpeakerV2Beep")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletPiezoSpeakerV2Beep"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletPiezoSpeakerV2Alarm")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletPiezoSpeakerV2Alarm"), RefreshType.REFRESH);
        }
    }

    public static void brickletPiezoSpeakerV2SetBeep(@Nullable ThingActions actions, int frequency, int volume,
            long duration) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2SetBeep(frequency, volume, duration);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Beep")
    public @ActionOutput(name = "frequency", type = "int") @ActionOutput(name = "volume", type = "int") @ActionOutput(name = "duration", type = "long") @ActionOutput(name = "durationRemaining", type = "long") Map<String, Object> brickletPiezoSpeakerV2GetBeep()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPiezoSpeakerV2Wrapper dev = ((@Nullable BrickletPiezoSpeakerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletPiezoSpeakerV2.Beep value = dev.getBeep();

        result.put("frequency", value.frequency);
        result.put("volume", value.volume);
        result.put("duration", value.duration);
        result.put("durationRemaining", value.durationRemaining);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetBeep(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetBeep();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Set Alarm")
    public void brickletPiezoSpeakerV2SetAlarm(@ActionInput(name = "startFrequency") int startFrequency,
            @ActionInput(name = "endFrequency") int endFrequency, @ActionInput(name = "stepSize") int stepSize,
            @ActionInput(name = "stepDelay") int stepDelay, @ActionInput(name = "volume") int volume,
            @ActionInput(name = "duration") long duration) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletPiezoSpeakerV2Wrapper dev = ((@Nullable BrickletPiezoSpeakerV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setAlarm(startFrequency, endFrequency, stepSize, stepDelay, volume, duration);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletPiezoSpeakerV2Beep")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletPiezoSpeakerV2Beep"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletPiezoSpeakerV2Alarm")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletPiezoSpeakerV2Alarm"), RefreshType.REFRESH);
        }
    }

    public static void brickletPiezoSpeakerV2SetAlarm(@Nullable ThingActions actions, int startFrequency,
            int endFrequency, int stepSize, int stepDelay, int volume, long duration) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2SetAlarm(startFrequency, endFrequency,
                    stepSize, stepDelay, volume, duration);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Alarm")
    public @ActionOutput(name = "startFrequency", type = "int") @ActionOutput(name = "endFrequency", type = "int") @ActionOutput(name = "stepSize", type = "int") @ActionOutput(name = "stepDelay", type = "int") @ActionOutput(name = "volume", type = "int") @ActionOutput(name = "duration", type = "long") @ActionOutput(name = "durationRemaining", type = "long") @ActionOutput(name = "currentFrequency", type = "int") Map<String, Object> brickletPiezoSpeakerV2GetAlarm()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPiezoSpeakerV2Wrapper dev = ((@Nullable BrickletPiezoSpeakerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletPiezoSpeakerV2.Alarm value = dev.getAlarm();

        result.put("startFrequency", value.startFrequency);
        result.put("endFrequency", value.endFrequency);
        result.put("stepSize", value.stepSize);
        result.put("stepDelay", value.stepDelay);
        result.put("volume", value.volume);
        result.put("duration", value.duration);
        result.put("durationRemaining", value.durationRemaining);
        result.put("currentFrequency", value.currentFrequency);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetAlarm(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetAlarm();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Update Volume")
    public void brickletPiezoSpeakerV2UpdateVolume(@ActionInput(name = "volume") int volume)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletPiezoSpeakerV2Wrapper dev = ((@Nullable BrickletPiezoSpeakerV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.updateVolume(volume);

    }

    public static void brickletPiezoSpeakerV2UpdateVolume(@Nullable ThingActions actions, int volume)
            throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2UpdateVolume(volume);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Update Frequency")
    public void brickletPiezoSpeakerV2UpdateFrequency(@ActionInput(name = "frequency") int frequency)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletPiezoSpeakerV2Wrapper dev = ((@Nullable BrickletPiezoSpeakerV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.updateFrequency(frequency);

    }

    public static void brickletPiezoSpeakerV2UpdateFrequency(@Nullable ThingActions actions, int frequency)
            throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2UpdateFrequency(frequency);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletPiezoSpeakerV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPiezoSpeakerV2Wrapper dev = ((@Nullable BrickletPiezoSpeakerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletPiezoSpeakerV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPiezoSpeakerV2Wrapper dev = ((@Nullable BrickletPiezoSpeakerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletPiezoSpeakerV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPiezoSpeakerV2Wrapper dev = ((@Nullable BrickletPiezoSpeakerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletPiezoSpeakerV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPiezoSpeakerV2Wrapper dev = ((@Nullable BrickletPiezoSpeakerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletPiezoSpeakerV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "int[]") @ActionOutput(name = "firmwareVersion", type = "int[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletPiezoSpeakerV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPiezoSpeakerV2Wrapper dev = ((@Nullable BrickletPiezoSpeakerV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletPiezoSpeakerV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }
}
